library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;

entity Spi_Master is
	generic (data_width: integer := 16);
    Port ( 	CLK  	: in   std_logic;
			RESET 	: in   std_logic;
			Datain	: in   std_logic_vector( data_width-1 DOWNTO 0);
			SCLK 	: out  std_logic;
			MOSI 	: out  std_logic;
			MISO 	: in   std_logic;
			CS1		: out  std_logic);  -- Slave Chip Select#1 
			-- CS		: out std_logic_vector (23 downto 0)
end Spi_Master;

architecture RTL of Spi_Master is

-----------------------------------------------------------
	-- -------- spi_Debounce Clock --------
-- component spi_clock
  -- Port (
		-- CLK_50 	: in  std_logic;
		-- x 		: in  std_logic;
		-- DBx 	: out std_logic
	  -- );
-- end component;
--------------------------------------------
-----------------------------------------------------------
signal shreg: std_logic_vector( data_width-1 DOWNTO 0);
type   states is (	Idle, Data_transfer, Stop);
signal State: states := Idle;


-- signal CLK_50 : std_logic;
-- signal x : std_logic;
	
	-- -------- spi_Debounce  --------
-- signal clk_signal 	: std_logic;

begin
-- ------------------------------------------------------
-- dut1 : spi_clock
-- port map (
			-- CLK_50  => CLK_50,  	
			-- x 	=>  x, 	
			-- DBx	=>  clk_signal );	-- DBx  : out  std_logic
									-- -- CLK  : in   std_logic
-- CLK <= clk_signal;

------------------------------------------------------
	MOSI <= shreg(shreg'left);
	SCLK <= CLK  when 
	State = Data_Transfer -- Master clock send to SCLK 
	else '0';                       
	
	process( CLK)
	variable Count: integer range 0 to data_width;
	begin
		if rising_edge( CLK) then
			case State is
				when Idle =>
				         CS1 <= '1';
							if RESET='1' then
								Count := 0;  			-- Init Data bit counter
								shreg <= Datain;  		-- Load parallel data
								CS1  <= '0';			-- Select Slave aktive
								State <= Data_Transfer; -- Change state
							end if;
				when Data_transfer =>
							if Count < (data_width-1) then 
								Count := Count+1;
								shreg <= shreg( shreg'left-1 downto 0)& MISO; 	-- Shift
							else
							   State <= Stop;			-- Change state
							end if;
				when Stop =>
							CS1 <= '1';		 			-- Deselect slave
							if RESET ='0' then
								State <= Idle;		
							end if;
			end case;
		end if;
	end process;
end RTL;



